`timescale 1ns / 1ps

/* Do NOT allow undeclared nets */
`default_nettype none

/*
Example for December 2014 - Programmable Logic In Practice.

Copyright Colin O'Flynn 2014. All Rights Reserved.
*/

module lx9_counter_test(
	 //Inputs
	 input wire 	USER_RESET,
	 input wire 	USER_CLOCK,
	 
	 //PMod Connection: J5, Pin 1
	 input wire 	PMOD1_P1, //1 MHz Clock
	  
	 //LEDs
	 output wire   GPIO_LED1
    );

	 wire reset;
	 wire inpclk;
	 	 
	 assign reset = USER_RESET;
	 
	 BUFG bufg_inst (
		.O(inpclk), // 1-bit output: Clock buffer output
		.I(USER_CLOCK) // 1-bit input: Clock buffer input
	 );
	 
	 reg [24:0] counterinp;
	 
	 assign GPIO_LED1 = counterinp[24];
		 
	 always @(posedge inpclk) begin
		counterinp <= counterinp + 25'd1;
	 end

 	 wire sysclk;
	 assign sysclk = inpclk;

	
	 reg [31:0] slowclk_cnt;
	 wire			slowclk;
	 
	 assign slowclk = PMOD1_P1;
	 
	 always @(posedge slowclk) begin
		slowclk_cnt <= slowclk_cnt + 32'd1;
	 end
	 
	 reg [31:0] cnt_buff1;
	 reg [31:0] cnt_buff2;
	  
	 wire [31:0] sysclk_cntinput;	 
	 
//`define USE_FIFO 1
	  
`ifdef USE_FIFO 
	 wire [31:0] fifoout;
	 wire full;
	 wire empty;
	 
	fifo_small_dualclock fifo (
	  .rst(reset), // input rst
	  .wr_clk(slowclk), // input wr_clk
	  .rd_clk(sysclk), // input rd_clk
	  .din(slowclk_cnt), // input [31 : 0] din
	  .wr_en(~full), // input wr_en
	  .rd_en(~empty), // input rd_en
	  .dout(fifoout), // output [31 : 0] dout
	  .full(full), // output full
	  .empty(empty) // output empty
	);
	assign sysclk_cntinput = fifoout;
`else
	assign sysclk_cntinput = slowclk_cnt;
`endif 
	 
	 always @(posedge sysclk) begin
		cnt_buff1 <= sysclk_cntinput; 
		cnt_buff2 <= cnt_buff1;
	 end
	 
	 reg [34:0] passcnt;
	 reg [24:0] failcnt;
	 
	 wire pass, fail;
	 assign pass = (cnt_buff1 == cnt_buff2) || (cnt_buff1 == (cnt_buff2 + 1)) ? 1 : 0;
	 assign fail = ~pass;
	 
	 always @(posedge sysclk) begin
		if (reset) begin
			passcnt <= 0;
			failcnt <= 0;
		end else if (pass)
			passcnt <= passcnt + 1;
		else
			failcnt <= failcnt + 1;
	 end 
	
	 wire [35:0] CONTROL0;
	 wire [59:0] debugdata;
	
	 assign debugdata[24:0] = failcnt;
	 assign debugdata[59:25] = passcnt;

	 debug_icon icon (
     .CONTROL0(CONTROL0) // INOUT BUS [35:0]
	 );
	
	 debug_ila ila (
     .CONTROL(CONTROL0), // INOUT BUS [35:0]
     .CLK(sysclk), // IN
     .TRIG0(debugdata) // IN BUS [59:0]
    );

	
	
	
	
endmodule
